﻿from Numeric import *
import types, os, getopt, sys

VersionNumber = "0.3.1"


def usage():
    print """
    yoochan@tiresias:~/classeur/python/klay$ python klay.py -c "corpus_simple.txt" -
    Klay.py calcule l'énergie nécessaire à la frappe d'un texte (corpus)
    sur un clavier donné, décrit par son fichier de configuration xkb.
    Le tout est pensé pour être hautement configurable, même si cette
    première version n'est qu'on prototype poussif

    SYNOPSIS
    python klay [options] --xkb xkb-config-file --corpus texte-latin9

    OPTIONS
    -k, --xkb
          xkb-config-file est l'extrait d'un
          fichier de configuration de type xkb

    -c, --corpus
          un fichier texte de référence, encodé
          en latin9 (par défaut : "corpus.txt")
          
    -v, --verbose
          mode bavard

    --version
    """

try:
    opts, args = getopt.getopt(sys.argv[1:], "hvk:c:", ["help", "verbose", "version", "xkb", "corpus"])
except getopt.GetoptError:
    # print help information and exit:
    usage()
    sys.exit(2)


corpusFile = r"corpus.txt"
XKBconfigFile = ""
verboseFlag = 0
for o, a in opts:
    if o in ("-h", "--help"):
        usage()
        sys.exit()
    if o in ("-k", "--xkb"):
        XKBconfigFile = a
    if o in ("-c", "--corpus"):
        corpusFile = a
    if o in ("-v", "--verbose"):
        verboseFlag = 1
    if o in ("--verbose"):
        print VersionNumber
        sys.exit()
try:
    fid_xkbConf = open(XKBconfigFile,'r')
except:
    usage()
    sys.exit()



print "Corpus : ", corpusFile
print "Fihier XKB : ", XKBconfigFile

charset_win1252 = {0x00 : 'NULL',
                   0x01 : 'START OF HEADING',
                   0x02 : 'START OF TEXT',
                   0x03 : 'END OF TEXT',
                   0x04 : 'END OF TRANSMISSION',
                   0x05 : 'ENQUIRY',
                   0x06 : 'ACKNOWLEDGE',
                   0x07 : 'BELL',
                   0x08 : 'BACKSPACE',
                   0x09 : 'HORIZONTAL TABULATION',
                   0x0A : 'LINE FEED',
                   0x0B : 'VERTICAL TABULATION',
                   0x0C : 'FORM FEED',
                   0x0D : 'CARRIAGE RETURN',
                   0x0E : 'SHIFT OUT',
                   0x0F : 'SHIFT IN',
                   0x10 : 'DATA LINK ESCAPE',
                   0x11 : 'DEVICE CONTROL ONE',
                   0x12 : 'DEVICE CONTROL TWO',
                   0x13 : 'DEVICE CONTROL THREE',
                   0x14 : 'DEVICE CONTROL FOUR',
                   0x15 : 'NEGATIVE ACKNOWLEDGE',
                   0x16 : 'SYNCHRONOUS IDLE',
                   0x17 : 'END OF TRANSMISSION BLOCK',
                   0x18 : 'CANCEL',
                   0x19 : 'END OF MEDIUM',
                   0x1A : 'SUBSTITUTE',
                   0x1B : 'ESCAPE',
                   0x1C : 'FILE SEPARATOR',
                   0x1D : 'GROUP SEPARATOR',
                   0x1E : 'RECORD SEPARATOR',
                   0x1F : 'UNIT SEPARATOR',
                   0x20 : 'SPACE',
                   0x21 : 'EXCLAMATION MARK',
                   0x22 : 'QUOTATION MARK',
                   0x23 : 'NUMBER SIGN',
                   0x24 : 'DOLLAR SIGN',
                   0x25 : 'PERCENT SIGN',
                   0x26 : 'AMPERSAND',
                   0x27 : 'APOSTROPHE',
                   0x28 : 'LEFT PARENTHESIS',
                   0x29 : 'RIGHT PARENTHESIS',
                   0x2A : 'ASTERISK',
                   0x2B : 'PLUS SIGN',
                   0x2C : 'COMMA',
                   0x2D : 'HYPHEN-MINUS',
                   0x2E : 'FULL STOP',
                   0x2F : 'SOLIDUS',
                   0x30 : 'DIGIT ZERO',
                   0x31 : 'DIGIT ONE',
                   0x32 : 'DIGIT TWO',
                   0x33 : 'DIGIT THREE',
                   0x34 : 'DIGIT FOUR',
                   0x35 : 'DIGIT FIVE',
                   0x36 : 'DIGIT SIX',
                   0x37 : 'DIGIT SEVEN',
                   0x38 : 'DIGIT EIGHT',
                   0x39 : 'DIGIT NINE',
                   0x3A : 'COLON',
                   0x3B : 'SEMICOLON',
                   0x3C : 'LESS-THAN SIGN',
                   0x3D : 'EQUALS SIGN',
                   0x3E : 'GREATER-THAN SIGN',
                   0x3F : 'QUESTION MARK',
                   0x40 : 'COMMERCIAL AT',
                   0x41 : 'LATIN CAPITAL LETTER A',
                   0x42 : 'LATIN CAPITAL LETTER B',
                   0x43 : 'LATIN CAPITAL LETTER C',
                   0x44 : 'LATIN CAPITAL LETTER D',
                   0x45 : 'LATIN CAPITAL LETTER E',
                   0x46 : 'LATIN CAPITAL LETTER F',
                   0x47 : 'LATIN CAPITAL LETTER G',
                   0x48 : 'LATIN CAPITAL LETTER H',
                   0x49 : 'LATIN CAPITAL LETTER I',
                   0x4A : 'LATIN CAPITAL LETTER J',
                   0x4B : 'LATIN CAPITAL LETTER K',
                   0x4C : 'LATIN CAPITAL LETTER L',
                   0x4D : 'LATIN CAPITAL LETTER M',
                   0x4E : 'LATIN CAPITAL LETTER N',
                   0x4F : 'LATIN CAPITAL LETTER O',
                   0x50 : 'LATIN CAPITAL LETTER P',
                   0x51 : 'LATIN CAPITAL LETTER Q',
                   0x52 : 'LATIN CAPITAL LETTER R',
                   0x53 : 'LATIN CAPITAL LETTER S',
                   0x54 : 'LATIN CAPITAL LETTER T',
                   0x55 : 'LATIN CAPITAL LETTER U',
                   0x56 : 'LATIN CAPITAL LETTER V',
                   0x57 : 'LATIN CAPITAL LETTER W',
                   0x58 : 'LATIN CAPITAL LETTER X',
                   0x59 : 'LATIN CAPITAL LETTER Y',
                   0x5A : 'LATIN CAPITAL LETTER Z',
                   0x5B : 'LEFT SQUARE BRACKET',
                   0x5C : 'REVERSE SOLIDUS',
                   0x5D : 'RIGHT SQUARE BRACKET',
                   0x5E : 'CIRCUMFLEX ACCENT',
                   0x5F : 'LOW LINE',
                   0x60 : 'GRAVE ACCENT',
                   0x61 : 'LATIN SMALL LETTER A',
                   0x62 : 'LATIN SMALL LETTER B',
                   0x63 : 'LATIN SMALL LETTER C',
                   0x64 : 'LATIN SMALL LETTER D',
                   0x65 : 'LATIN SMALL LETTER E',
                   0x66 : 'LATIN SMALL LETTER F',
                   0x67 : 'LATIN SMALL LETTER G',
                   0x68 : 'LATIN SMALL LETTER H',
                   0x69 : 'LATIN SMALL LETTER I',
                   0x6A : 'LATIN SMALL LETTER J',
                   0x6B : 'LATIN SMALL LETTER K',
                   0x6C : 'LATIN SMALL LETTER L',
                   0x6D : 'LATIN SMALL LETTER M',
                   0x6E : 'LATIN SMALL LETTER N',
                   0x6F : 'LATIN SMALL LETTER O',
                   0x70 : 'LATIN SMALL LETTER P',
                   0x71 : 'LATIN SMALL LETTER Q',
                   0x72 : 'LATIN SMALL LETTER R',
                   0x73 : 'LATIN SMALL LETTER S',
                   0x74 : 'LATIN SMALL LETTER T',
                   0x75 : 'LATIN SMALL LETTER U',
                   0x76 : 'LATIN SMALL LETTER V',
                   0x77 : 'LATIN SMALL LETTER W',
                   0x78 : 'LATIN SMALL LETTER X',
                   0x79 : 'LATIN SMALL LETTER Y',
                   0x7A : 'LATIN SMALL LETTER Z',
                   0x7B : 'LEFT CURLY BRACKET',
                   0x7C : 'VERTICAL LINE',
                   0x7D : 'RIGHT CURLY BRACKET',
                   0x7E : 'TILDE',
                   0x7F : 'DELETE',
                   0x80 : 'EURO SIGN',
                   0x81 : 'UNDEFINED',
                   0x82 : 'SINGLE LOW-9 QUOTATION MARK',
                   0x83 : 'LATIN SMALL LETTER F WITH HOOK',
                   0x84 : 'DOUBLE LOW-9 QUOTATION MARK',
                   0x85 : 'HORIZONTAL ELLIPSIS',
                   0x86 : 'DAGGER',
                   0x87 : 'DOUBLE DAGGER',
                   0x88 : 'MODIFIER LETTER CIRCUMFLEX ACCENT',
                   0x89 : 'PER MILLE SIGN',
                   0x8A : 'LATIN CAPITAL LETTER S WITH CARON',
                   0x8B : 'SINGLE LEFT-POINTING ANGLE QUOTATION MARK',
                   0x8C : 'LATIN CAPITAL LIGATURE OE',
                   0x8D : 'UNDEFINED',
                   0x8E : 'LATIN CAPITAL LETTER Z WITH CARON',
                   0x8F : 'UNDEFINED',
                   0x90 : 'UNDEFINED',
                   0x91 : 'LEFT SINGLE QUOTATION MARK',
                   0x92 : 'RIGHT SINGLE QUOTATION MARK',
                   0x93 : 'LEFT DOUBLE QUOTATION MARK',
                   0x94 : 'RIGHT DOUBLE QUOTATION MARK',
                   0x95 : 'BULLET',
                   0x96 : 'EN DASH',
                   0x97 : 'EM DASH',
                   0x98 : 'SMALL TILDE',
                   0x99 : 'TRADE MARK SIGN',
                   0x9A : 'LATIN SMALL LETTER S WITH CARON',
                   0x9B : 'SINGLE RIGHT-POINTING ANGLE QUOTATION MARK',
                   0x9C : 'LATIN SMALL LIGATURE OE',
                   0x9D : 'UNDEFINED',
                   0x9E : 'LATIN SMALL LETTER Z WITH CARON',
                   0x9F : 'LATIN CAPITAL LETTER Y WITH DIAERESIS',
                   0xA0 : 'NO-BREAK SPACE',
                   0xA1 : 'INVERTED EXCLAMATION MARK',
                   0xA2 : 'CENT SIGN',
                   0xA3 : 'POUND SIGN',
                   0xA4 : 'CURRENCY SIGN',
                   0xA5 : 'YEN SIGN',
                   0xA6 : 'BROKEN BAR',
                   0xA7 : 'SECTION SIGN',
                   0xA8 : 'DIAERESIS',
                   0xA9 : 'COPYRIGHT SIGN',
                   0xAA : 'FEMININE ORDINAL INDICATOR',
                   0xAB : 'LEFT-POINTING DOUBLE ANGLE QUOTATION MARK',
                   0xAC : 'NOT SIGN',
                   0xAD : 'SOFT HYPHEN',
                   0xAE : 'REGISTERED SIGN',
                   0xAF : 'MACRON',
                   0xB0 : 'DEGREE SIGN',
                   0xB1 : 'PLUS-MINUS SIGN',
                   0xB2 : 'SUPERSCRIPT TWO',
                   0xB3 : 'SUPERSCRIPT THREE',
                   0xB4 : 'ACUTE ACCENT',
                   0xB5 : 'MICRO SIGN',
                   0xB6 : 'PILCROW SIGN',
                   0xB7 : 'MIDDLE DOT',
                   0xB8 : 'CEDILLA',
                   0xB9 : 'SUPERSCRIPT ONE',
                   0xBA : 'MASCULINE ORDINAL INDICATOR',
                   0xBB : 'RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK',
                   0xBC : 'VULGAR FRACTION ONE QUARTER',
                   0xBD : 'VULGAR FRACTION ONE HALF',
                   0xBE : 'VULGAR FRACTION THREE QUARTERS',
                   0xBF : 'INVERTED QUESTION MARK',
                   0xC0 : 'LATIN CAPITAL LETTER A WITH GRAVE',
                   0xC1 : 'LATIN CAPITAL LETTER A WITH ACUTE',
                   0xC2 : 'LATIN CAPITAL LETTER A WITH CIRCUMFLEX',
                   0xC3 : 'LATIN CAPITAL LETTER A WITH TILDE',
                   0xC4 : 'LATIN CAPITAL LETTER A WITH DIAERESIS',
                   0xC5 : 'LATIN CAPITAL LETTER A WITH RING ABOVE',
                   0xC6 : 'LATIN CAPITAL LIGATURE AE',
                   0xC7 : 'LATIN CAPITAL LETTER C WITH CEDILLA',
                   0xC8 : 'LATIN CAPITAL LETTER E WITH GRAVE',
                   0xC9 : 'LATIN CAPITAL LETTER E WITH ACUTE',
                   0xCA : 'LATIN CAPITAL LETTER E WITH CIRCUMFLEX',
                   0xCB : 'LATIN CAPITAL LETTER E WITH DIAERESIS',
                   0xCC : 'LATIN CAPITAL LETTER I WITH GRAVE',
                   0xCD : 'LATIN CAPITAL LETTER I WITH ACUTE',
                   0xCE : 'LATIN CAPITAL LETTER I WITH CIRCUMFLEX',
                   0xCF : 'LATIN CAPITAL LETTER I WITH DIAERESIS',
                   0xD0 : 'LATIN CAPITAL LETTER ETH',
                   0xD1 : 'LATIN CAPITAL LETTER N WITH TILDE',
                   0xD2 : 'LATIN CAPITAL LETTER O WITH GRAVE',
                   0xD3 : 'LATIN CAPITAL LETTER O WITH ACUTE',
                   0xD4 : 'LATIN CAPITAL LETTER O WITH CIRCUMFLEX',
                   0xD5 : 'LATIN CAPITAL LETTER O WITH TILDE',
                   0xD6 : 'LATIN CAPITAL LETTER O WITH DIAERESIS',
                   0xD7 : 'MULTIPLICATION SIGN',
                   0xD8 : 'LATIN CAPITAL LETTER O WITH STROKE',
                   0xD9 : 'LATIN CAPITAL LETTER U WITH GRAVE',
                   0xDA : 'LATIN CAPITAL LETTER U WITH ACUTE',
                   0xDB : 'LATIN CAPITAL LETTER U WITH CIRCUMFLEX',
                   0xDC : 'LATIN CAPITAL LETTER U WITH DIAERESIS',
                   0xDD : 'LATIN CAPITAL LETTER Y WITH ACUTE',
                   0xDE : 'LATIN CAPITAL LETTER THORN',
                   0xDF : 'LATIN SMALL LETTER SHARP S',
                   0xE0 : 'LATIN SMALL LETTER A WITH GRAVE',
                   0xE1 : 'LATIN SMALL LETTER A WITH ACUTE',
                   0xE2 : 'LATIN SMALL LETTER A WITH CIRCUMFLEX',
                   0xE3 : 'LATIN SMALL LETTER A WITH TILDE',
                   0xE4 : 'LATIN SMALL LETTER A WITH DIAERESIS',
                   0xE5 : 'LATIN SMALL LETTER A WITH RING ABOVE',
                   0xE6 : 'LATIN SMALL LIGATURE AE',
                   0xE7 : 'LATIN SMALL LETTER C WITH CEDILLA',
                   0xE8 : 'LATIN SMALL LETTER E WITH GRAVE',
                   0xE9 : 'LATIN SMALL LETTER E WITH ACUTE',
                   0xEA : 'LATIN SMALL LETTER E WITH CIRCUMFLEX',
                   0xEB : 'LATIN SMALL LETTER E WITH DIAERESIS',
                   0xEC : 'LATIN SMALL LETTER I WITH GRAVE',
                   0xED : 'LATIN SMALL LETTER I WITH ACUTE',
                   0xEE : 'LATIN SMALL LETTER I WITH CIRCUMFLEX',
                   0xEF : 'LATIN SMALL LETTER I WITH DIAERESIS',
                   0xF0 : 'LATIN SMALL LETTER ETH',
                   0xF1 : 'LATIN SMALL LETTER N WITH TILDE',
                   0xF2 : 'LATIN SMALL LETTER O WITH GRAVE',
                   0xF3 : 'LATIN SMALL LETTER O WITH ACUTE',
                   0xF4 : 'LATIN SMALL LETTER O WITH CIRCUMFLEX',
                   0xF5 : 'LATIN SMALL LETTER O WITH TILDE',
                   0xF6 : 'LATIN SMALL LETTER O WITH DIAERESIS',
                   0xF7 : 'DIVISION SIGN',
                   0xF8 : 'LATIN SMALL LETTER O WITH STROKE',
                   0xF9 : 'LATIN SMALL LETTER U WITH GRAVE',
                   0xFA : 'LATIN SMALL LETTER U WITH ACUTE',
                   0xFB : 'LATIN SMALL LETTER U WITH CIRCUMFLEX',
                   0xFC : 'LATIN SMALL LETTER U WITH DIAERESIS',
                   0xFD : 'LATIN SMALL LETTER Y WITH ACUTE',
                   0xFE : 'LATIN SMALL LETTER THORN',
                   0xFF : 'LATIN SMALL LETTER Y WITH DIAERESIS'}
                               
charset_latin1 = {0x00 : 'NULL',
                  0x01 : 'START OF HEADING',
                  0x02 : 'START OF TEXT',
                  0x03 : 'END OF TEXT',
                  0x04 : 'END OF TRANSMISSION',
                  0x05 : 'ENQUIRY',
                  0x06 : 'ACKNOWLEDGE',
                  0x07 : 'BELL',
                  0x08 : 'BACKSPACE',
                  0x09 : 'HORIZONTAL TABULATION',
                  0x0A : 'LINE FEED',
                  0x0B : 'VERTICAL TABULATION',
                  0x0C : 'FORM FEED',
                  0x0D : 'CARRIAGE RETURN',
                  0x0E : 'SHIFT OUT',
                  0x0F : 'SHIFT IN',
                  0x10 : 'DATA LINK ESCAPE',
                  0x11 : 'DEVICE CONTROL ONE',
                  0x12 : 'DEVICE CONTROL TWO',
                  0x13 : 'DEVICE CONTROL THREE',
                  0x14 : 'DEVICE CONTROL FOUR',
                  0x15 : 'NEGATIVE ACKNOWLEDGE',
                  0x16 : 'SYNCHRONOUS IDLE',
                  0x17 : 'END OF TRANSMISSION BLOCK',
                  0x18 : 'CANCEL',
                  0x19 : 'END OF MEDIUM',
                  0x1A : 'SUBSTITUTE',
                  0x1B : 'ESCAPE',
                  0x1C : 'FILE SEPARATOR',
                  0x1D : 'GROUP SEPARATOR',
                  0x1E : 'RECORD SEPARATOR',
                  0x1F : 'UNIT SEPARATOR',
                  0x20 : 'SPACE',
                  0x21 : 'EXCLAMATION MARK',
                  0x22 : 'QUOTATION MARK',
                  0x23 : 'NUMBER SIGN',
                  0x24 : 'DOLLAR SIGN',
                  0x25 : 'PERCENT SIGN',
                  0x26 : 'AMPERSAND',
                  0x27 : 'APOSTROPHE',
                  0x28 : 'LEFT PARENTHESIS',
                  0x29 : 'RIGHT PARENTHESIS',
                  0x2A : 'ASTERISK',
                  0x2B : 'PLUS SIGN',
                  0x2C : 'COMMA',
                  0x2D : 'HYPHEN-MINUS',
                  0x2E : 'FULL STOP',
                  0x2F : 'SOLIDUS',
                  0x30 : 'DIGIT ZERO',
                  0x31 : 'DIGIT ONE',
                  0x32 : 'DIGIT TWO',
                  0x33 : 'DIGIT THREE',
                  0x34 : 'DIGIT FOUR',
                  0x35 : 'DIGIT FIVE',
                  0x36 : 'DIGIT SIX',
                  0x37 : 'DIGIT SEVEN',
                  0x38 : 'DIGIT EIGHT',
                  0x39 : 'DIGIT NINE',
                  0x3A : 'COLON',
                  0x3B : 'SEMICOLON',
                  0x3C : 'LESS-THAN SIGN',
                  0x3D : 'EQUALS SIGN',
                  0x3E : 'GREATER-THAN SIGN',
                  0x3F : 'QUESTION MARK',
                  0x40 : 'COMMERCIAL AT',
                  0x41 : 'LATIN CAPITAL LETTER A',
                  0x42 : 'LATIN CAPITAL LETTER B',
                  0x43 : 'LATIN CAPITAL LETTER C',
                  0x44 : 'LATIN CAPITAL LETTER D',
                  0x45 : 'LATIN CAPITAL LETTER E',
                  0x46 : 'LATIN CAPITAL LETTER F',
                  0x47 : 'LATIN CAPITAL LETTER G',
                  0x48 : 'LATIN CAPITAL LETTER H',
                  0x49 : 'LATIN CAPITAL LETTER I',
                  0x4A : 'LATIN CAPITAL LETTER J',
                  0x4B : 'LATIN CAPITAL LETTER K',
                  0x4C : 'LATIN CAPITAL LETTER L',
                  0x4D : 'LATIN CAPITAL LETTER M',
                  0x4E : 'LATIN CAPITAL LETTER N',
                  0x4F : 'LATIN CAPITAL LETTER O',
                  0x50 : 'LATIN CAPITAL LETTER P',
                  0x51 : 'LATIN CAPITAL LETTER Q',
                  0x52 : 'LATIN CAPITAL LETTER R',
                  0x53 : 'LATIN CAPITAL LETTER S',
                  0x54 : 'LATIN CAPITAL LETTER T',
                  0x55 : 'LATIN CAPITAL LETTER U',
                  0x56 : 'LATIN CAPITAL LETTER V',
                  0x57 : 'LATIN CAPITAL LETTER W',
                  0x58 : 'LATIN CAPITAL LETTER X',
                  0x59 : 'LATIN CAPITAL LETTER Y',
                  0x5A : 'LATIN CAPITAL LETTER Z',
                  0x5B : 'LEFT SQUARE BRACKET',
                  0x5C : 'REVERSE SOLIDUS',
                  0x5D : 'RIGHT SQUARE BRACKET',
                  0x5E : 'CIRCUMFLEX ACCENT',
                  0x5F : 'LOW LINE',
                  0x60 : 'GRAVE ACCENT',
                  0x61 : 'LATIN SMALL LETTER A',
                  0x62 : 'LATIN SMALL LETTER B',
                  0x63 : 'LATIN SMALL LETTER C',
                  0x64 : 'LATIN SMALL LETTER D',
                  0x65 : 'LATIN SMALL LETTER E',
                  0x66 : 'LATIN SMALL LETTER F',
                  0x67 : 'LATIN SMALL LETTER G',
                  0x68 : 'LATIN SMALL LETTER H',
                  0x69 : 'LATIN SMALL LETTER I',
                  0x6A : 'LATIN SMALL LETTER J',
                  0x6B : 'LATIN SMALL LETTER K',
                  0x6C : 'LATIN SMALL LETTER L',
                  0x6D : 'LATIN SMALL LETTER M',
                  0x6E : 'LATIN SMALL LETTER N',
                  0x6F : 'LATIN SMALL LETTER O',
                  0x70 : 'LATIN SMALL LETTER P',
                  0x71 : 'LATIN SMALL LETTER Q',
                  0x72 : 'LATIN SMALL LETTER R',
                  0x73 : 'LATIN SMALL LETTER S',
                  0x74 : 'LATIN SMALL LETTER T',
                  0x75 : 'LATIN SMALL LETTER U',
                  0x76 : 'LATIN SMALL LETTER V',
                  0x77 : 'LATIN SMALL LETTER W',
                  0x78 : 'LATIN SMALL LETTER X',
                  0x79 : 'LATIN SMALL LETTER Y',
                  0x7A : 'LATIN SMALL LETTER Z',
                  0x7B : 'LEFT CURLY BRACKET',
                  0x7C : 'VERTICAL LINE',
                  0x7D : 'RIGHT CURLY BRACKET',
                  0x7E : 'TILDE',
                  0x7F : 'UNDEFINED',
                  0x80 : 'UNDEFINED',
                  0x81 : 'UNDEFINED',
                  0x82 : 'UNDEFINED',
                  0x83 : 'UNDEFINED',
                  0x84 : 'UNDEFINED',
                  0x85 : 'UNDEFINED',
                  0x86 : 'UNDEFINED',
                  0x87 : 'UNDEFINED',
                  0x88 : 'UNDEFINED',
                  0x89 : 'UNDEFINED',
                  0x8A : 'UNDEFINED',
                  0x8B : 'UNDEFINED',
                  0x8C : 'UNDEFINED',
                  0x8D : 'UNDEFINED',
                  0x8E : 'UNDEFINED',
                  0x8F : 'UNDEFINED',
                  0x90 : 'UNDEFINED',
                  0x91 : 'UNDEFINED',
                  0x92 : 'UNDEFINED',
                  0x93 : 'UNDEFINED',
                  0x94 : 'UNDEFINED',
                  0x95 : 'UNDEFINED',
                  0x96 : 'UNDEFINED',
                  0x97 : 'UNDEFINED',
                  0x98 : 'UNDEFINED',
                  0x99 : 'UNDEFINED',
                  0x9A : 'UNDEFINED',
                  0x9B : 'UNDEFINED',
                  0x9C : 'UNDEFINED',
                  0x9D : 'UNDEFINED',
                  0x9E : 'UNDEFINED',
                  0x9F : 'UNDEFINED',
                  0xA0 : 'NO-BREAK SPACE',
                  0xA1 : 'INVERTED EXCLAMATION MARK',
                  0xA2 : 'CENT SIGN',
                  0xA3 : 'POUND SIGN',
                  0xA4 : 'CURRENCY SIGN',
                  0xA5 : 'YEN SIGN',
                  0xA6 : 'BROKEN BAR',
                  0xA7 : 'SECTION SIGN',
                  0xA8 : 'DIAERESIS',
                  0xA9 : 'COPYRIGHT SIGN',
                  0xAA : 'FEMININE ORDINAL INDICATOR',
                  0xAB : 'LEFT-POINTING DOUBLE ANGLE QUOTATION MARK',
                  0xAC : 'NOT SIGN',
                  0xAD : 'SOFT HYPHEN',
                  0xAE : 'REGISTERED SIGN',
                  0xAF : 'MACRON',
                  0xB0 : 'DEGREE SIGN',
                  0xB1 : 'PLUS-MINUS SIGN',
                  0xB2 : 'SUPERSCRIPT TWO',
                  0xB3 : 'SUPERSCRIPT THREE',
                  0xB4 : 'ACUTE ACCENT',
                  0xB5 : 'MICRO SIGN',
                  0xB6 : 'PILCROW SIGN',
                  0xB7 : 'MIDDLE DOT',
                  0xB8 : 'CEDILLA',
                  0xB9 : 'SUPERSCRIPT ONE',
                  0xBA : 'MASCULINE ORDINAL INDICATOR',
                  0xBB : 'RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK',
                  0xBC : 'VULGAR FRACTION ONE QUARTER',
                  0xBD : 'VULGAR FRACTION ONE HALF',
                  0xBE : 'VULGAR FRACTION THREE QUARTERS',
                  0xBF : 'INVERTED QUESTION MARK',
                  0xC0 : 'LATIN CAPITAL LETTER A WITH GRAVE',
                  0xC1 : 'LATIN CAPITAL LETTER A WITH ACUTE',
                  0xC2 : 'LATIN CAPITAL LETTER A WITH CIRCUMFLEX',
                  0xC3 : 'LATIN CAPITAL LETTER A WITH TILDE',
                  0xC4 : 'LATIN CAPITAL LETTER A WITH DIAERESIS',
                  0xC5 : 'LATIN CAPITAL LETTER A WITH RING ABOVE',
                  0xC6 : 'LATIN CAPITAL LIGATURE AE',
                  0xC7 : 'LATIN CAPITAL LETTER C WITH CEDILLA',
                  0xC8 : 'LATIN CAPITAL LETTER E WITH GRAVE',
                  0xC9 : 'LATIN CAPITAL LETTER E WITH ACUTE',
                  0xCA : 'LATIN CAPITAL LETTER E WITH CIRCUMFLEX',
                  0xCB : 'LATIN CAPITAL LETTER E WITH DIAERESIS',
                  0xCC : 'LATIN CAPITAL LETTER I WITH GRAVE',
                  0xCD : 'LATIN CAPITAL LETTER I WITH ACUTE',
                  0xCE : 'LATIN CAPITAL LETTER I WITH CIRCUMFLEX',
                  0xCF : 'LATIN CAPITAL LETTER I WITH DIAERESIS',
                  0xD0 : 'LATIN CAPITAL LETTER ETH',
                  0xD1 : 'LATIN CAPITAL LETTER N WITH TILDE',
                  0xD2 : 'LATIN CAPITAL LETTER O WITH GRAVE',
                  0xD3 : 'LATIN CAPITAL LETTER O WITH ACUTE',
                  0xD4 : 'LATIN CAPITAL LETTER O WITH CIRCUMFLEX',
                  0xD5 : 'LATIN CAPITAL LETTER O WITH TILDE',
                  0xD6 : 'LATIN CAPITAL LETTER O WITH DIAERESIS',
                  0xD7 : 'MULTIPLICATION SIGN',
                  0xD8 : 'LATIN CAPITAL LETTER O WITH STROKE',
                  0xD9 : 'LATIN CAPITAL LETTER U WITH GRAVE',
                  0xDA : 'LATIN CAPITAL LETTER U WITH ACUTE',
                  0xDB : 'LATIN CAPITAL LETTER U WITH CIRCUMFLEX',
                  0xDC : 'LATIN CAPITAL LETTER U WITH DIAERESIS',
                  0xDD : 'LATIN CAPITAL LETTER Y WITH ACUTE',
                  0xDE : 'LATIN CAPITAL LETTER THORN',
                  0xDF : 'LATIN SMALL LETTER SHARP S',
                  0xE0 : 'LATIN SMALL LETTER A WITH GRAVE',
                  0xE1 : 'LATIN SMALL LETTER A WITH ACUTE',
                  0xE2 : 'LATIN SMALL LETTER A WITH CIRCUMFLEX',
                  0xE3 : 'LATIN SMALL LETTER A WITH TILDE',
                  0xE4 : 'LATIN SMALL LETTER A WITH DIAERESIS',
                  0xE5 : 'LATIN SMALL LETTER A WITH RING ABOVE',
                  0xE6 : 'LATIN SMALL LIGATURE AE',
                  0xE7 : 'LATIN SMALL LETTER C WITH CEDILLA',
                  0xE8 : 'LATIN SMALL LETTER E WITH GRAVE',
                  0xE9 : 'LATIN SMALL LETTER E WITH ACUTE',
                  0xEA : 'LATIN SMALL LETTER E WITH CIRCUMFLEX',
                  0xEB : 'LATIN SMALL LETTER E WITH DIAERESIS',
                  0xEC : 'LATIN SMALL LETTER I WITH GRAVE',
                  0xED : 'LATIN SMALL LETTER I WITH ACUTE',
                  0xEE : 'LATIN SMALL LETTER I WITH CIRCUMFLEX',
                  0xEF : 'LATIN SMALL LETTER I WITH DIAERESIS',
                  0xF0 : 'LATIN SMALL LETTER ETH',
                  0xF1 : 'LATIN SMALL LETTER N WITH TILDE',
                  0xF2 : 'LATIN SMALL LETTER O WITH GRAVE',
                  0xF3 : 'LATIN SMALL LETTER O WITH ACUTE',
                  0xF4 : 'LATIN SMALL LETTER O WITH CIRCUMFLEX',
                  0xF5 : 'LATIN SMALL LETTER O WITH TILDE',
                  0xF6 : 'LATIN SMALL LETTER O WITH DIAERESIS',
                  0xF7 : 'DIVISION SIGN',
                  0xF8 : 'LATIN SMALL LETTER O WITH STROKE',
                  0xF9 : 'LATIN SMALL LETTER U WITH GRAVE',
                  0xFA : 'LATIN SMALL LETTER U WITH ACUTE',
                  0xFB : 'LATIN SMALL LETTER U WITH CIRCUMFLEX',
                  0xFC : 'LATIN SMALL LETTER U WITH DIAERESIS',
                  0xFD : 'LATIN SMALL LETTER Y WITH ACUTE',
                  0xFE : 'LATIN SMALL LETTER THORN',
                  0xFF : 'LATIN SMALL LETTER Y WITH DIAERESIS'}
                   
charset_latin9 = {0x00 : 'NULL',
                   0x01 : 'START OF HEADING',
                   0x02 : 'START OF TEXT',
                   0x03 : 'END OF TEXT',
                   0x04 : 'END OF TRANSMISSION',
                   0x05 : 'ENQUIRY',
                   0x06 : 'ACKNOWLEDGE',
                   0x07 : 'BELL',
                   0x08 : 'BACKSPACE',
                   0x09 : 'HORIZONTAL TABULATION',
                   0x0A : 'LINE FEED',
                   0x0B : 'VERTICAL TABULATION',
                   0x0C : 'FORM FEED',
                   0x0D : 'CARRIAGE RETURN',
                   0x0E : 'SHIFT OUT',
                   0x0F : 'SHIFT IN',
                   0x10 : 'DATA LINK ESCAPE',
                   0x11 : 'DEVICE CONTROL ONE',
                   0x12 : 'DEVICE CONTROL TWO',
                   0x13 : 'DEVICE CONTROL THREE',
                   0x14 : 'DEVICE CONTROL FOUR',
                   0x15 : 'NEGATIVE ACKNOWLEDGE',
                   0x16 : 'SYNCHRONOUS IDLE',
                   0x17 : 'END OF TRANSMISSION BLOCK',
                   0x18 : 'CANCEL',
                   0x19 : 'END OF MEDIUM',
                   0x1A : 'SUBSTITUTE',
                   0x1B : 'ESCAPE',
                   0x1C : 'FILE SEPARATOR',
                   0x1D : 'GROUP SEPARATOR',
                   0x1E : 'RECORD SEPARATOR',
                   0x1F : 'UNIT SEPARATOR',
                   0x20 : 'SPACE',
                   0x21 : 'EXCLAMATION MARK',
                   0x22 : 'QUOTATION MARK',
                   0x23 : 'NUMBER SIGN',
                   0x24 : 'DOLLAR SIGN',
                   0x25 : 'PERCENT SIGN',
                   0x26 : 'AMPERSAND',
                   0x27 : 'APOSTROPHE',
                   0x28 : 'LEFT PARENTHESIS',
                   0x29 : 'RIGHT PARENTHESIS',
                   0x2A : 'ASTERISK',
                   0x2B : 'PLUS SIGN',
                   0x2C : 'COMMA',
                   0x2D : 'HYPHEN-MINUS',
                   0x2E : 'FULL STOP',
                   0x2F : 'SOLIDUS',
                   0x30 : 'DIGIT ZERO',
                   0x31 : 'DIGIT ONE',
                   0x32 : 'DIGIT TWO',
                   0x33 : 'DIGIT THREE',
                   0x34 : 'DIGIT FOUR',
                   0x35 : 'DIGIT FIVE',
                   0x36 : 'DIGIT SIX',
                   0x37 : 'DIGIT SEVEN',
                   0x38 : 'DIGIT EIGHT',
                   0x39 : 'DIGIT NINE',
                   0x3A : 'COLON',
                   0x3B : 'SEMICOLON',
                   0x3C : 'LESS-THAN SIGN',
                   0x3D : 'EQUALS SIGN',
                   0x3E : 'GREATER-THAN SIGN',
                   0x3F : 'QUESTION MARK',
                   0x40 : 'COMMERCIAL AT',
                   0x41 : 'LATIN CAPITAL LETTER A',
                   0x42 : 'LATIN CAPITAL LETTER B',
                   0x43 : 'LATIN CAPITAL LETTER C',
                   0x44 : 'LATIN CAPITAL LETTER D',
                   0x45 : 'LATIN CAPITAL LETTER E',
                   0x46 : 'LATIN CAPITAL LETTER F',
                   0x47 : 'LATIN CAPITAL LETTER G',
                   0x48 : 'LATIN CAPITAL LETTER H',
                   0x49 : 'LATIN CAPITAL LETTER I',
                   0x4A : 'LATIN CAPITAL LETTER J',
                   0x4B : 'LATIN CAPITAL LETTER K',
                   0x4C : 'LATIN CAPITAL LETTER L',
                   0x4D : 'LATIN CAPITAL LETTER M',
                   0x4E : 'LATIN CAPITAL LETTER N',
                   0x4F : 'LATIN CAPITAL LETTER O',
                   0x50 : 'LATIN CAPITAL LETTER P',
                   0x51 : 'LATIN CAPITAL LETTER Q',
                   0x52 : 'LATIN CAPITAL LETTER R',
                   0x53 : 'LATIN CAPITAL LETTER S',
                   0x54 : 'LATIN CAPITAL LETTER T',
                   0x55 : 'LATIN CAPITAL LETTER U',
                   0x56 : 'LATIN CAPITAL LETTER V',
                   0x57 : 'LATIN CAPITAL LETTER W',
                   0x58 : 'LATIN CAPITAL LETTER X',
                   0x59 : 'LATIN CAPITAL LETTER Y',
                   0x5A : 'LATIN CAPITAL LETTER Z',
                   0x5B : 'LEFT SQUARE BRACKET',
                   0x5C : 'REVERSE SOLIDUS',
                   0x5D : 'RIGHT SQUARE BRACKET',
                   0x5E : 'CIRCUMFLEX ACCENT',
                   0x5F : 'LOW LINE',
                   0x60 : 'GRAVE ACCENT',
                   0x61 : 'LATIN SMALL LETTER A',
                   0x62 : 'LATIN SMALL LETTER B',
                   0x63 : 'LATIN SMALL LETTER C',
                   0x64 : 'LATIN SMALL LETTER D',
                   0x65 : 'LATIN SMALL LETTER E',
                   0x66 : 'LATIN SMALL LETTER F',
                   0x67 : 'LATIN SMALL LETTER G',
                   0x68 : 'LATIN SMALL LETTER H',
                   0x69 : 'LATIN SMALL LETTER I',
                   0x6A : 'LATIN SMALL LETTER J',
                   0x6B : 'LATIN SMALL LETTER K',
                   0x6C : 'LATIN SMALL LETTER L',
                   0x6D : 'LATIN SMALL LETTER M',
                   0x6E : 'LATIN SMALL LETTER N',
                   0x6F : 'LATIN SMALL LETTER O',
                   0x70 : 'LATIN SMALL LETTER P',
                   0x71 : 'LATIN SMALL LETTER Q',
                   0x72 : 'LATIN SMALL LETTER R',
                   0x73 : 'LATIN SMALL LETTER S',
                   0x74 : 'LATIN SMALL LETTER T',
                   0x75 : 'LATIN SMALL LETTER U',
                   0x76 : 'LATIN SMALL LETTER V',
                   0x77 : 'LATIN SMALL LETTER W',
                   0x78 : 'LATIN SMALL LETTER X',
                   0x79 : 'LATIN SMALL LETTER Y',
                   0x7A : 'LATIN SMALL LETTER Z',
                   0x7B : 'LEFT CURLY BRACKET',
                   0x7C : 'VERTICAL LINE',
                   0x7D : 'RIGHT CURLY BRACKET',
                   0x7E : 'TILDE',
                   0x7F : 'UNDEFINED',
                   0x80 : 'UNDEFINED',
                   0x81 : 'UNDEFINED',
                   0x82 : 'UNDEFINED',
                   0x83 : 'UNDEFINED',
                   0x84 : 'UNDEFINED',
                   0x85 : 'UNDEFINED',
                   0x86 : 'UNDEFINED',
                   0x87 : 'UNDEFINED',
                   0x88 : 'UNDEFINED',
                   0x89 : 'UNDEFINED',
                   0x8A : 'UNDEFINED',
                   0x8B : 'UNDEFINED',
                   0x8C : 'UNDEFINED',
                   0x8D : 'UNDEFINED',
                   0x8E : 'UNDEFINED',
                   0x8F : 'UNDEFINED',
                   0x90 : 'UNDEFINED',
                   0x91 : 'UNDEFINED',
                   0x92 : 'UNDEFINED',
                   0x93 : 'UNDEFINED',
                   0x94 : 'UNDEFINED',
                   0x95 : 'UNDEFINED',
                   0x96 : 'UNDEFINED',
                   0x97 : 'UNDEFINED',
                   0x98 : 'UNDEFINED',
                   0x99 : 'UNDEFINED',
                   0x9A : 'UNDEFINED',
                   0x9B : 'UNDEFINED',
                   0x9C : 'UNDEFINED',
                   0x9D : 'UNDEFINED',
                   0x9E : 'UNDEFINED',
                   0x9F : 'UNDEFINED',
                   0xA0 : 'NO-BREAK SPACE',
                   0xA1 : 'INVERTED EXCLAMATION MARK',
                   0xA2 : 'CENT SIGN',
                   0xA3 : 'POUND SIGN',
                   0xA4 : 'EURO SIGN',
                   0xA5 : 'YEN SIGN',
                   0xA6 : 'LATIN CAPITAL LETTER S WITH CARON',
                   0xA7 : 'SECTION SIGN',
                   0xA8 : 'LATIN SMALL LETTER S WITH CARON',
                   0xA9 : 'COPYRIGHT SIGN',
                   0xAA : 'FEMININE ORDINAL INDICATOR',
                   0xAB : 'LEFT-POINTING DOUBLE ANGLE QUOTATION MARK',
                   0xAC : 'NOT SIGN',
                   0xAD : 'SOFT HYPHEN',
                   0xAE : 'REGISTERED SIGN',
                   0xAF : 'MACRON',
                   0xB0 : 'DEGREE SIGN',
                   0xB1 : 'PLUS-MINUS SIGN',
                   0xB2 : 'SUPERSCRIPT TWO',
                   0xB3 : 'SUPERSCRIPT THREE',
                   0xB4 : 'LATIN CAPITAL LETTER Z WITH CARON',
                   0xB5 : 'MICRO SIGN',
                   0xB6 : 'PILCROW SIGN',
                   0xB7 : 'MIDDLE DOT',
                   0xB8 : 'LATIN SMALL LETTER Z WITH CARON',
                   0xB9 : 'SUPERSCRIPT ONE',
                   0xBA : 'MASCULINE ORDINAL INDICATOR',
                   0xBB : 'RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK',
                   0xBC : 'LATIN CAPITAL LIGATURE OE',
                   0xBD : 'LATIN SMALL LIGATURE OE',
                   0xBE : 'LATIN CAPITAL LETTER Y WITH DIAERESIS',
                   0xBF : 'INVERTED QUESTION MARK',
                   0xC0 : 'LATIN CAPITAL LETTER A WITH GRAVE',
                   0xC1 : 'LATIN CAPITAL LETTER A WITH ACUTE',
                   0xC2 : 'LATIN CAPITAL LETTER A WITH CIRCUMFLEX',
                   0xC3 : 'LATIN CAPITAL LETTER A WITH TILDE',
                   0xC4 : 'LATIN CAPITAL LETTER A WITH DIAERESIS',
                   0xC5 : 'LATIN CAPITAL LETTER A WITH RING ABOVE',
                   0xC6 : 'LATIN CAPITAL LIGATURE AE',
                   0xC7 : 'LATIN CAPITAL LETTER C WITH CEDILLA',
                   0xC8 : 'LATIN CAPITAL LETTER E WITH GRAVE',
                   0xC9 : 'LATIN CAPITAL LETTER E WITH ACUTE',
                   0xCA : 'LATIN CAPITAL LETTER E WITH CIRCUMFLEX',
                   0xCB : 'LATIN CAPITAL LETTER E WITH DIAERESIS',
                   0xCC : 'LATIN CAPITAL LETTER I WITH GRAVE',
                   0xCD : 'LATIN CAPITAL LETTER I WITH ACUTE',
                   0xCE : 'LATIN CAPITAL LETTER I WITH CIRCUMFLEX',
                   0xCF : 'LATIN CAPITAL LETTER I WITH DIAERESIS',
                   0xD0 : 'LATIN CAPITAL LETTER ETH',
                   0xD1 : 'LATIN CAPITAL LETTER N WITH TILDE',
                   0xD2 : 'LATIN CAPITAL LETTER O WITH GRAVE',
                   0xD3 : 'LATIN CAPITAL LETTER O WITH ACUTE',
                   0xD4 : 'LATIN CAPITAL LETTER O WITH CIRCUMFLEX',
                   0xD5 : 'LATIN CAPITAL LETTER O WITH TILDE',
                   0xD6 : 'LATIN CAPITAL LETTER O WITH DIAERESIS',
                   0xD7 : 'MULTIPLICATION SIGN',
                   0xD8 : 'LATIN CAPITAL LETTER O WITH STROKE',
                   0xD9 : 'LATIN CAPITAL LETTER U WITH GRAVE',
                   0xDA : 'LATIN CAPITAL LETTER U WITH ACUTE',
                   0xDB : 'LATIN CAPITAL LETTER U WITH CIRCUMFLEX',
                   0xDC : 'LATIN CAPITAL LETTER U WITH DIAERESIS',
                   0xDD : 'LATIN CAPITAL LETTER Y WITH ACUTE',
                   0xDE : 'LATIN CAPITAL LETTER THORN',
                   0xDF : 'LATIN SMALL LETTER SHARP S',
                   0xE0 : 'LATIN SMALL LETTER A WITH GRAVE',
                   0xE1 : 'LATIN SMALL LETTER A WITH ACUTE',
                   0xE2 : 'LATIN SMALL LETTER A WITH CIRCUMFLEX',
                   0xE3 : 'LATIN SMALL LETTER A WITH TILDE',
                   0xE4 : 'LATIN SMALL LETTER A WITH DIAERESIS',
                   0xE5 : 'LATIN SMALL LETTER A WITH RING ABOVE',
                   0xE6 : 'LATIN SMALL LIGATURE AE',
                   0xE7 : 'LATIN SMALL LETTER C WITH CEDILLA',
                   0xE8 : 'LATIN SMALL LETTER E WITH GRAVE',
                   0xE9 : 'LATIN SMALL LETTER E WITH ACUTE',
                   0xEA : 'LATIN SMALL LETTER E WITH CIRCUMFLEX',
                   0xEB : 'LATIN SMALL LETTER E WITH DIAERESIS',
                   0xEC : 'LATIN SMALL LETTER I WITH GRAVE',
                   0xED : 'LATIN SMALL LETTER I WITH ACUTE',
                   0xEE : 'LATIN SMALL LETTER I WITH CIRCUMFLEX',
                   0xEF : 'LATIN SMALL LETTER I WITH DIAERESIS',
                   0xF0 : 'LATIN SMALL LETTER ETH',
                   0xF1 : 'LATIN SMALL LETTER N WITH TILDE',
                   0xF2 : 'LATIN SMALL LETTER O WITH GRAVE',
                   0xF3 : 'LATIN SMALL LETTER O WITH ACUTE',
                   0xF4 : 'LATIN SMALL LETTER O WITH CIRCUMFLEX',
                   0xF5 : 'LATIN SMALL LETTER O WITH TILDE',
                   0xF6 : 'LATIN SMALL LETTER O WITH DIAERESIS',
                   0xF7 : 'DIVISION SIGN',
                   0xF8 : 'LATIN SMALL LETTER O WITH STROKE',
                   0xF9 : 'LATIN SMALL LETTER U WITH GRAVE',
                   0xFA : 'LATIN SMALL LETTER U WITH ACUTE',
                   0xFB : 'LATIN SMALL LETTER U WITH CIRCUMFLEX',
                   0xFC : 'LATIN SMALL LETTER U WITH DIAERESIS',
                   0xFD : 'LATIN SMALL LETTER Y WITH ACUTE',
                   0xFE : 'LATIN SMALL LETTER THORN',
                   0xFF : 'LATIN SMALL LETTER Y WITH DIAERESIS'}

layout_bridge = {'NULL' : [('')],
                 'START OF HEADING' : [('')],
                 'START OF TEXT' : [('')],
                 'END OF TEXT' : [('')],
                 'END OF TRANSMISSION' : [('')],
                 'ENQUIRY' : [('')],
                 'ACKNOWLEDGE' : [('')],
                 'BELL' : [('')],
                 'BACKSPACE' : [('')],
                 'HORIZONTAL TABULATION' : [('')],
                 'LINE FEED' : [('return')],
                 'VERTICAL TABULATION' : [('')],
                 'FORM FEED' : [('')],
                 'CARRIAGE RETURN' : [('return')],
                 'SHIFT OUT' : [('')],
                 'SHIFT IN' : [('')],
                 'DATA LINK ESCAPE' : [('')],
                 'DEVICE CONTROL ONE' : [('')],
                 'DEVICE CONTROL TWO' : [('')],
                 'DEVICE CONTROL THREE' : [('')],
                 'DEVICE CONTROL FOUR' : [('')],
                 'NEGATIVE ACKNOWLEDGE' : [('')],
                 'SYNCHRONOUS IDLE' : [('')],
                 'END OF TRANSMISSION BLOCK' : [('')],
                 'CANCEL' : [('')],
                 'END OF MEDIUM' : [('')],
                 'SUBSTITUTE' : [('')],
                 'ESCAPE' : [('')],
                 'FILE SEPARATOR' : [('')],
                 'GROUP SEPARATOR' : [('')],
                 'RECORD SEPARATOR' : [('')],
                 'UNIT SEPARATOR' : [('')],
                 'SPACE' : [('space')],
                 'EXCLAMATION MARK' : [('exclam')],
                 'QUOTATION MARK' : [('quotedbl')],
                 'NUMBER SIGN' : [('numbersign')],
                 'DOLLAR SIGN' : [('dollar')],
                 'PERCENT SIGN' : [('percent')],
                 'AMPERSAND' : [('ampersand')],
                 'APOSTROPHE' : [('apostrophe')],
                 'LEFT PARENTHESIS' : [('braceleft')],
                 'RIGHT PARENTHESIS' : [('braceright')],
                 'ASTERISK' : [('asterisk')],
                 'PLUS SIGN' : [('plus')],
                 'COMMA' : [('comma')],
                 'HYPHEN-MINUS' : [('minus')],
                 'FULL STOP' : [('period')],
                 'SOLIDUS' : [('slash')],
                 'DIGIT ZERO' : [('0')],
                 'DIGIT ONE' : [('1')],
                 'DIGIT TWO' : [('2')],
                 'DIGIT THREE' : [('3')],
                 'DIGIT FOUR' : [('4')],
                 'DIGIT FIVE' : [('5')],
                 'DIGIT SIX' : [('6')],
                 'DIGIT SEVEN' : [('7')],
                 'DIGIT EIGHT' : [('8')],
                 'DIGIT NINE' : [('9')],
                 'COLON' : [('colon')],
                 'SEMICOLON' : [('semicolon')],
                 'LESS-THAN SIGN' : [('less')],
                 'EQUALS SIGN' : [('equal')],
                 'GREATER-THAN SIGN' : [('greater')],
                 'QUESTION MARK' : [('question')],
                 'COMMERCIAL AT' : [('at')],
                 'LATIN CAPITAL LETTER A' : [('A')],
                 'LATIN CAPITAL LETTER B' : [('B')],
                 'LATIN CAPITAL LETTER C' : [('C')],
                 'LATIN CAPITAL LETTER D' : [('D')],
                 'LATIN CAPITAL LETTER E' : [('E')],
                 'LATIN CAPITAL LETTER F' : [('F')],
                 'LATIN CAPITAL LETTER G' : [('G')],
                 'LATIN CAPITAL LETTER H' : [('H')],
                 'LATIN CAPITAL LETTER I' : [('I')],
                 'LATIN CAPITAL LETTER J' : [('J')],
                 'LATIN CAPITAL LETTER K' : [('K')],
                 'LATIN CAPITAL LETTER L' : [('L')],
                 'LATIN CAPITAL LETTER M' : [('M')],
                 'LATIN CAPITAL LETTER N' : [('N')],
                 'LATIN CAPITAL LETTER O' : [('O')],
                 'LATIN CAPITAL LETTER P' : [('P')],
                 'LATIN CAPITAL LETTER Q' : [('Q')],
                 'LATIN CAPITAL LETTER R' : [('R')],
                 'LATIN CAPITAL LETTER S' : [('S')],
                 'LATIN CAPITAL LETTER T' : [('T')],
                 'LATIN CAPITAL LETTER U' : [('U')],
                 'LATIN CAPITAL LETTER V' : [('V')],
                 'LATIN CAPITAL LETTER W' : [('W')],
                 'LATIN CAPITAL LETTER X' : [('X')],
                 'LATIN CAPITAL LETTER Y' : [('Y')],
                 'LATIN CAPITAL LETTER Z' : [('Z')],
                 'LEFT SQUARE BRACKET' : [('bracketleft')],
                 'REVERSE SOLIDUS' : [('backslash')],
                 'RIGHT SQUARE BRACKET' : [('bracketright')],
                 'CIRCUMFLEX ACCENT' : [('asciicircum')],
                 'LOW LINE' : [('underscore')],
                 'GRAVE ACCENT' : [('grave')],
                 'LATIN SMALL LETTER A' : [('a')],
                 'LATIN SMALL LETTER B' : [('b')],
                 'LATIN SMALL LETTER C' : [('c')],
                 'LATIN SMALL LETTER D' : [('d')],
                 'LATIN SMALL LETTER E' : [('e')],
                 'LATIN SMALL LETTER F' : [('f')],
                 'LATIN SMALL LETTER G' : [('g')],
                 'LATIN SMALL LETTER H' : [('h')],
                 'LATIN SMALL LETTER I' : [('i')],
                 'LATIN SMALL LETTER J' : [('j')],
                 'LATIN SMALL LETTER K' : [('k')],
                 'LATIN SMALL LETTER L' : [('l')],
                 'LATIN SMALL LETTER M' : [('m')],
                 'LATIN SMALL LETTER N' : [('n')],
                 'LATIN SMALL LETTER O' : [('o')],
                 'LATIN SMALL LETTER P' : [('p')],
                 'LATIN SMALL LETTER Q' : [('q')],
                 'LATIN SMALL LETTER R' : [('r')],
                 'LATIN SMALL LETTER S' : [('s')],
                 'LATIN SMALL LETTER T' : [('t')],
                 'LATIN SMALL LETTER U' : [('u')],
                 'LATIN SMALL LETTER V' : [('v')],
                 'LATIN SMALL LETTER W' : [('w')],
                 'LATIN SMALL LETTER X' : [('x')],
                 'LATIN SMALL LETTER Y' : [('y')],
                 'LATIN SMALL LETTER Z' : [('z')],
                 'LEFT CURLY BRACKET' : [('braceleft')],
                 'VERTICAL LINE' : [('bar')],
                 'RIGHT CURLY BRACKET' : [('braceright')],
                 'TILDE' : [('asciitilde')],
                 'NO-BREAK SPACE' : [('nobreakspace')],
                 'INVERTED EXCLAMATION MARK' : [('exclamdown')],
                 'CENT SIGN' : [('cent')],
                 'POUND SIGN' : [('sterling')],
                 'EURO SIGN' : [('EuroSign')],
                 'YEN SIGN' : [('yen')],
                 'LATIN CAPITAL LETTER S WITH CARON' : [('dead_abovering','S')],
                 'SECTION SIGN' : [('section')],
                 'LATIN SMALL LETTER S WITH CARON' : [('dead_abovering','s')],
                 'COPYRIGHT SIGN' : [('copyright')],
                 'FEMININE ORDINAL INDICATOR' : [('ordfeminine')],
                 'LEFT-POINTING DOUBLE ANGLE QUOTATION MARK' : [('guillemotleft')],
                 'NOT SIGN' : [('notsign')],
                 'SOFT HYPHEN' : [('minus')],
                 'REGISTERED SIGN' : [('registered')],
                 'MACRON' : [('macron')],
                 'DEGREE SIGN' : [('degree')],
                 'PLUS-MINUS SIGN' : [('plusminus')],
                 'SUPERSCRIPT TWO' : [('twosuperior')],
                 'SUPERSCRIPT THREE' : [('threesuperior')],
                 'LATIN CAPITAL LETTER Z WITH CARON' : [('dead_abovering','Z')],
                 'MICRO SIGN' : [('mu')],
                 'PILCROW SIGN' : [('paragraph')],
                 'MIDDLE DOT' : [('periodcentered')],
                 'LATIN SMALL LETTER Z WITH CARON' : [('dead_abovering','z')],
                 'SUPERSCRIPT ONE' : [('onesuperior')],
                 'MASCULINE ORDINAL INDICATOR' : [('masculine')],
                 'RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK' : [('guillemotright')],
                 'LATIN CAPITAL LIGATURE OE' : [('OE')],
                 'LATIN SMALL LIGATURE OE' : [('oe')],
                 'LATIN CAPITAL LETTER Y WITH DIAERESIS' : [('dead_diaeresis','Y')],
                 'INVERTED QUESTION MARK' : [('questiondown')],
                 'LATIN CAPITAL LETTER A WITH GRAVE' : [('Agrave')],
                 'LATIN CAPITAL LETTER A WITH ACUTE' : [('dead_acute','A')],
                 'LATIN CAPITAL LETTER A WITH CIRCUMFLEX' : [('dead_circumflex','A')],
                 'LATIN CAPITAL LETTER A WITH TILDE' : [('dead_tilde','A')],
                 'LATIN CAPITAL LETTER A WITH DIAERESIS' : [('dead_diaeresis','A')],
                 'LATIN CAPITAL LETTER A WITH RING ABOVE' : [('dead_abovering','A')],
                 'LATIN CAPITAL LIGATURE AE' : [('AE')],
                 'LATIN CAPITAL LETTER C WITH CEDILLA' : [('Ccedilla'),('dead_cedilla','C')],
                 'LATIN CAPITAL LETTER E WITH GRAVE' : [('Egrave'),('dead_grave','E')],
                 'LATIN CAPITAL LETTER E WITH ACUTE' : [('Eacute')],
                 'LATIN CAPITAL LETTER E WITH CIRCUMFLEX' : [('dead_circumflex','E')],
                 'LATIN CAPITAL LETTER E WITH DIAERESIS' : [('dead_diaeresis','E')],
                 'LATIN CAPITAL LETTER I WITH GRAVE' : [('dead_grave','I')],
                 'LATIN CAPITAL LETTER I WITH ACUTE' : [('dead_acute','I')],
                 'LATIN CAPITAL LETTER I WITH CIRCUMFLEX' : [('dead_circumflex','I')],
                 'LATIN CAPITAL LETTER I WITH DIAERESIS' : [('dead_diaeresis','I')],
                 'LATIN CAPITAL LETTER ETH' : [('ETH')],
                 'LATIN CAPITAL LETTER N WITH TILDE' : [('dead_tilde','N')],
                 'LATIN CAPITAL LETTER O WITH GRAVE' : [('dead_grave','O')],
                 'LATIN CAPITAL LETTER O WITH ACUTE' : [('dead_acute','O')],
                 'LATIN CAPITAL LETTER O WITH CIRCUMFLEX' : [('dead_circumflex','O')],
                 'LATIN CAPITAL LETTER O WITH TILDE' : [('dead_tilde','O')],
                 'LATIN CAPITAL LETTER O WITH DIAERESIS' : [('dead_diaeresis','O')],
                 'MULTIPLICATION SIGN' : [('multiply')],
                 'LATIN CAPITAL LETTER O WITH STROKE' : [('Ooblique')],
                 'LATIN CAPITAL LETTER U WITH GRAVE' : [('Ugrave')],
                 'LATIN CAPITAL LETTER U WITH ACUTE' : [('dead_acute','U')],
                 'LATIN CAPITAL LETTER U WITH CIRCUMFLEX' : [('dead_circumflex','U')],
                 'LATIN CAPITAL LETTER U WITH DIAERESIS' : [('dead_diaeresis','u')],
                 'LATIN CAPITAL LETTER Y WITH ACUTE' : [('dead_acute','Y')],
                 'LATIN CAPITAL LETTER THORN' : [('Thorn')],
                 'LATIN SMALL LETTER SHARP S' : [('ssharp')],
                 'LATIN SMALL LETTER A WITH GRAVE' : [('agrave')],
                 'LATIN SMALL LETTER A WITH ACUTE' : [('dead_acute','A')],
                 'LATIN SMALL LETTER A WITH CIRCUMFLEX' : [('dead_circumflex','a')],
                 'LATIN SMALL LETTER A WITH TILDE' : [('dead_tilde','a')],
                 'LATIN SMALL LETTER A WITH DIAERESIS' : [('dead_diaeresis','a')],
                 'LATIN SMALL LETTER A WITH RING ABOVE' : [('dead_abovering','a')],
                 'LATIN SMALL LIGATURE AE' : [('ae')],
                 'LATIN SMALL LETTER C WITH CEDILLA' : [('ccedilla')],
                 'LATIN SMALL LETTER E WITH GRAVE' : [('egrave')],
                 'LATIN SMALL LETTER E WITH ACUTE' : [('eacute')],
                 'LATIN SMALL LETTER E WITH CIRCUMFLEX' : [('dead_circumflex','e')],
                 'LATIN SMALL LETTER E WITH DIAERESIS' : [('dead_diaeresis','e')],
                 'LATIN SMALL LETTER I WITH GRAVE' : [('dead_grave','i')],
                 'LATIN SMALL LETTER I WITH ACUTE' : [('dead_acute','i')],
                 'LATIN SMALL LETTER I WITH CIRCUMFLEX' : [('dead_circumflex','i')],
                 'LATIN SMALL LETTER I WITH DIAERESIS' : [('dead_diaeresis','i')],
                 'LATIN SMALL LETTER ETH' : [('eth')],
                 'LATIN SMALL LETTER N WITH TILDE' : [('dead_tilde','n')],
                 'LATIN SMALL LETTER O WITH GRAVE' : [('dead_grave','o')],
                 'LATIN SMALL LETTER O WITH ACUTE' : [('dead_acute','o')],
                 'LATIN SMALL LETTER O WITH CIRCUMFLEX' : [('dead_circumflex','o')],
                 'LATIN SMALL LETTER O WITH TILDE' : [('dead_tilde','o')],
                 'LATIN SMALL LETTER O WITH DIAERESIS' : [('dead_diaeresis','o')],
                 'DIVISION SIGN' : [('division')],
                 'LATIN SMALL LETTER O WITH STROKE' : [('oslash')],
                 'LATIN SMALL LETTER U WITH GRAVE' : [('ugrave')],
                 'LATIN SMALL LETTER U WITH ACUTE' : [('dead_acute','u')],
                 'LATIN SMALL LETTER U WITH CIRCUMFLEX' : [('dead_circumflex','u')],
                 'LATIN SMALL LETTER U WITH DIAERESIS' : [('dead_diaeresis','u')],
                 'LATIN SMALL LETTER Y WITH ACUTE' : [('dead_acute','y')],
                 'LATIN SMALL LETTER THORN' : [('thorn')],
                 'LATIN SMALL LETTER Y WITH DIAERESIS' : [('dead_diaeresis','y')]}

# AE01:['g',2,-0.5,+2]
# touche AE01 main gauche, deuxieme doigt (majeur), position X = -0.5 en Y = +2
keymap = {'TLDE' : ['l',4,-11,+10],
          'AE01' : ['l',4,-5,+10],
          'AE02' : ['l',3,+1,+10],
          'AE03' : ['l',3,+1,+10],
          'AE04' : ['l',2,+1,+10],
          'AE05' : ['l',2,+1,+10],
          'AE06' : ['l',1,+7,+10],
          'AE07' : ['r',1,-5,+10],
          'AE08' : ['r',2,+1,+10],
          'AE09' : ['r',3,+1,+10],
          'AE10' : ['r',4,+1,+10],
          'AE11' : ['r',4,+1,+10],
          'AE12' : ['r',4,+7,+10],
          'AD01' : ['l',4,-2,+5],
          'AD02' : ['l',3,-2,+5],
          'AD03' : ['l',2,-2,+5],
          'AD04' : ['l',1,-2,+5],
          'AD05' : ['l',1,+4,+5],
          'AD06' : ['r',1,-8,+5],
          'AD07' : ['r',1,-2,+5],
          'AD08' : ['r',2,-2,+5],
          'AD09' : ['r',3,-2,+5],
          'AD10' : ['r',4,-2,+5],
          'AD11' : ['r',4,+4,+5],
          'AD12' : ['r',4,+10,+5],
          'AC01' : ['l',4,0,0],
          'AC02' : ['l',3,0,0],
          'AC03' : ['l',2,0,0],
          'AC04' : ['l',1,0,0],
          'AC05' : ['l',1,+6,0],
          'AC06' : ['r',1,-6,0],
          'AC07' : ['r',1,0,0],
          'AC08' : ['r',2,0,0],
          'AC09' : ['r',3,0,0],
          'AC10' : ['r',4,0,0],
          'AC11' : ['r',4,+6,0],
          'BKSL' : ['r',4,+12,0],
          'LSGT' : ['l',4,-3,-5],
          'AB01' : ['l',4,+3,-5],
          'AB02' : ['l',3,+3,-5],
          'AB03' : ['l',1,+3,-5],
          'AB04' : ['l',1,+3,-5],
          'AB05' : ['l',1,+9,-5],
          'AB06' : ['r',1,-3,-5],
          'AB07' : ['r',1,+3,-5],
          'AB08' : ['r',2,+3,-5],
          'AB09' : ['r',3,+3,-5],
          'AB10' : ['r',4,+3,-5],
          'SPCE' : ['lr',0,0,-5],
          'RTRN' : ['r',4,0,+18],
          'LFSH' : ['l',4,-9,-5],
          'RTSH' : ['r',4,+5,+5],
          'RALT' : ['r',0,+10,-5]}

class virtualHand:
    def __init__(self,handside):
        self.handSide = handside
        self.handPos = array([0.0,0.0])
        self.handStiffness = 5
        self.handEnergy = 0
        self.fingerPos = array([[0,0],[0,0],[0,0],[0,0],[0,0]])
        self.fingerStiffness = array([1,0.8,0.9,1,1.2])
        self.fingerEnergy = 0
        self.keyPressed = 0
        self.energy = 0.0
        
    def resetFingers(self):
        self.fingerPos = array([[0,0],[0,0],[0,0],[0,0],[0,0]])
        self.handPos = array([0.0,0.0])
    
    def moveFinger(self,newKey,keymap):
        self.keyPressed += 1
        fingerNo = keymap[newKey][1]
        prevPos = self.fingerPos[fingerNo]
        newPos = keymap[newKey][2:]
        fingerEnergy = newPos[0]**2 - prevPos[0]**2 + newPos[1]**2 - prevPos[1]**2
        self.fingerPos[fingerNo] = newPos
        
        self.fingerEnergy = 0.5 * self.fingerStiffness[fingerNo] * fingerEnergy
        
        prevPos = self.handPos
        newPos = array([sum(self.fingerPos[:,0] * self.fingerStiffness) / 5.0,
                       sum(self.fingerPos[:,1] * self.fingerStiffness) / 5.0])
        handEnergy = newPos[0]**2 - prevPos[0]**2 + newPos[1]**2 - prevPos[1]**2
        self.handPos = newPos
        
        self.handEnergy = 0.5 * self.handStiffness * handEnergy
        self.energy += self.handEnergy + self.fingerEnergy
        # print self.handEnergy + self.fingerEnergy ## debug ##

def parseXkbConf(fid_xkbConf,keymap):
    import re
    
    str_catchKey = r"<([A-Z0-9]{4})>.*\[(.*)\]"
    re_catchKey = re.compile(str_catchKey)
    
    revert = dict()
    
    for line in fid_xkbConf.readlines():
        m = re_catchKey.search(line)
        if m:
            codeXkb = str(m.group(1))
            lineXkb = str(m.group(2))
            
            column = 0
            for labelXkb in re.split(',',lineXkb):
                labelXkb = labelXkb.strip()
                if column == 0:
                    keyCombinaison = codeXkb
                elif column == 1:
                    if keymap[codeXkb][0] == 'l':
                        keyCombinaison = ('RTSH',codeXkb)
                    else:
                        keyCombinaison = ('LFSH',codeXkb)
                elif column == 2:
                    keyCombinaison = ('RALT',codeXkb)
                elif column == 3:
                    keyCombinaison = ('RALT','RTSH', codeXkb)
                    
                if labelXkb in revert:
                    revert[labelXkb].append(keyCombinaison)
                else:
                    revert[labelXkb] = [keyCombinaison]
                column = column + 1
    return revert

physicalLayout = parseXkbConf(fid_xkbConf,keymap)

def yieldSymbolSequence(chrIn,
                        charset = charset_latin9,
                        bridge = layout_bridge,
                        keyboard = physicalLayout):
    
    labelUnicode = charset[ord(chrIn)]
    
    tokenAltern = False
    
    if labelUnicode in bridge:
        for labelSequence in bridge[labelUnicode]:
            if not tokenAltern:
                if type(labelSequence)==types.TupleType:
                    tokenSequence = True
                    for label in labelSequence:
                        if label not in keyboard:
                            tokenSequence = False
                    if tokenSequence:
                        for label in labelSequence:
                            tokenAltern = True
                            yield label
                else:
                    if labelSequence in keyboard:
                        tokenAltern = True
                        yield labelSequence
    
def yieldKeySequence(labelIn, charset = charset_latin9, bridge = layout_bridge):
    tokenAltern = False
    for keySequence in physicalLayout[labelIn]:
        if not tokenAltern:
            if type(keySequence)==types.TupleType:
                tokenSequence = True
                if tokenSequence:
                    for key in keySequence:
                        tokenAltern = True
                        yield key
            else:
                tokenAltern = True
                yield keySequence

def prepareByteToKey():
    byteToKey = dict()
    for byte in range(1,256):
        keySequence = []
        for symbol in yieldSymbolSequence(chr(byte)):
            for key in yieldKeySequence(symbol):
                keySequence.append(key)
        byteToKey[byte] = keySequence
    return byteToKey

def yieldKeyFromByte(dictKey,byte):
    for key in dictKey[byte]:
        yield key

def printByteToKey(dictKey, charset, fid):
    for byte in dictKey:
        fid.write(`byte` + " " + chr(max([byte,32])) + " - "+str(charset[byte])+" : "+ str(dictKey[byte])+"\n")

leftHand = virtualHand('l')
rightHand = virtualHand('r')


fid_corpus = open(corpusFile,'r')
fid_log = open('dictkey.txt','w')
fid_dictkey = open('listing.txt','w')

chrCurs = fid_corpus.read(1)

while chrCurs:
    fid_dictkey.write(str(chrCurs)+'\n')
    chrCurs = fid_corpus.read(1)
fid_dictkey.close()

dict_latin9 = prepareByteToKey()

printByteToKey(dict_latin9,charset_latin9,fid_log)

fid_corpus.seek(0)

fullText = array(fromstring(fid_corpus.read(),'c'),'c')

print "\nAnalyse en cours ...\n"


for chrCurs in fullText:
    keylog = ""
    for key in yieldKeyFromByte(dict_latin9,ord(str(chrCurs))):
        if keymap[key][0] == 'r':
            keylog += key + " "
            leftHand.resetFingers()
            rightHand.moveFinger(key,keymap)
        if keymap[key][0] == 'l':
            keylog += key + " "
            rightHand.resetFingers()
            leftHand.moveFinger(key,keymap)
        else :
            rightHand.resetFingers()
            leftHand.resetFingers()
    if verboseFlag :
        print chrCurs, keylog

fid_log.close()
fid_corpus.close()
textLen = (len(fullText)-1)
print "nombre de caractères : " , textLen
print "                                 main gauche / main droite"
print "       Énergie totale dépensée : ", leftHand.energy, " / ",  rightHand.energy
print "    Nombre de touches pressées : ", leftHand.keyPressed, " / ", rightHand.keyPressed
print "   Énergie dépensée par touche : ", leftHand.energy / leftHand.keyPressed , " / ", rightHand.energy / rightHand.keyPressed

print "\net hop !"
